/*
 * postdata.c
 */

#include "postdata.h"
#include "setup.h"
#include "audiorecorder.h"
#include "mpa-frame.h"
#include "a-tools.h"

#include <string.h>

/* --- cPostData ------------------------------------------------------------ */

cPostData::cPostData(void)
{
        title = NULL;
        artist = NULL;
        genre = NULL;
        comment = NULL;
        path = NULL;
        new_path = NULL;
        
        codec = NULL;
        
        next = NULL;
}


cPostData::~cPostData()
{
        reset();
}


cPostData::cPostData(const cPostData *postdata)
{
        title = strdup0(postdata->get_title());
        artist = strdup0(postdata->get_artist());
        genre = strdup0(postdata->get_genre());
        comment = strdup0(postdata->get_comment());
        
        codec = strdup0(postdata->get_codec());
        bit_rate = postdata->get_bit_rate();
        sample_rate = postdata->get_sample_rate();
        channels = postdata->get_channels();
        frame_len = postdata->get_frame_len();
        
        len_mpa_frame = postdata->get_len_mpa_frame();
        
        fade_in_mode = postdata->get_fade_in_mode();
        frames_fade_in = postdata->get_frames_fade_in();
        fade_out_mode = postdata->get_fade_out_mode();
        frames_fade_out = postdata->get_frames_fade_out();
        
        path = strdup0(postdata->get_path());
        new_path = strdup0(postdata->get_new_path());
        
        next = NULL;
}


void cPostData::new_track(char *_path, char *_comment)
{
        reset();
        
        path = strdup0(_path);
        comment = strdup0(_comment);
        
        if (cPluginAudiorecorder::get_dbg_level() > 1)
                printf("--- comment set to: <%s> ---\n", comment);
}


void cPostData::stop_track(int _bit_rate, int _sample_rate, int _channels,
        int _len_mpa_frame)
{
        sample_rate = _sample_rate;
        channels = _channels;
        len_mpa_frame = _len_mpa_frame;
        
        if (SetupValues.audio_codec == 0) {
                codec = strdup("mp2");
                bit_rate = _bit_rate;
        }
        else {
                codec = strdup(audio_codecs[SetupValues.audio_codec]);
                bit_rate = bit_rates[SetupValues.bit_rate + 1];
        }
        
        /* TODO: adjust this for other codecs then mp2/mp3 */
        frame_len = 144 * bit_rate / sample_rate;
        
        set_fade_values();
        set_new_path();
}


void cPostData::set_title(char *_title)
{
        FREE(title);
        title = strdup0(_title);
        
        if (cPluginAudiorecorder::get_dbg_level() > 1)
                printf("--- title set to: <%s> ---\n", title);
}


void cPostData::set_artist(char *_artist)
{
        FREE(artist);
        artist = strdup0(_artist);
        
        if (cPluginAudiorecorder::get_dbg_level() > 1)
                printf("--- artist set to: <%s> ---\n", artist);
}


void cPostData::set_genre(char *_genre)
{
        FREE(genre);
        genre = strdup0(_genre);
        printf("--- genre set to: <%s> ---\n", genre);        
}


void cPostData::set_genre(int pty)
{
        /* map rds pty-codes 10-15 and 24-28 to id3v1 tags */
        static char *ptys[29] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL,
                NULL, NULL, NULL, "Pop", "Rock", "Easy Listening", "Classical",
                "Classical", "Other", NULL, NULL, NULL, NULL, NULL, NULL, NULL,
                NULL, "Jazz", "Country", "Folklore", "Oldies", "Folk" };
                
        if ( (pty > 9 && pty < 16) || (pty > 23 && pty < 29) ) {
                FREE(genre);
                genre = strdup0(ptys[pty]);
                
                if (cPluginAudiorecorder::get_dbg_level() > 1)
                        printf("--- genre set to: <%s> ---\n", genre);
        }
}


void cPostData::reset()
{
        FREE(title);
        FREE(artist);
        FREE(comment);
        FREE(genre);
        FREE(codec);
        FREE(path);
        FREE(new_path);
}


void cPostData::set_fade_values()
{
        fade_in_mode = SetupValues.fade_in_mode;
        if (fade_in_mode == 0)
                frames_fade_in = 0;
        else
                frames_fade_in = SetupValues.fade_in * (bit_rate / 8 /
                        frame_len);
        
        fade_out_mode = SetupValues.fade_out_mode;
        if (fade_out_mode == 0)
                frames_fade_out = 0;
        else
                frames_fade_out = SetupValues.fade_out * (bit_rate / 8 /
                        frame_len);
}


void cPostData::set_new_path()
{
        char *_artist, *_title;
        _artist = strdup0(artist);
        _title = strdup0(title);
        strtr(_artist);
        strtr(_title);
        
  	asprintf(&new_path, "%s/%s-%s.%s", cPluginAudiorecorder::get_recdir(),
                _artist, _title, codec);
        FREE(_artist);
        FREE(_title);
}
