/*
 * postproc.c
 */

/* this is needed for gcc 2.95 */
#include <vector>

#include "postproc.h"
#include "setup.h"
#include "a-tools.h"

#include <vdr/tools.h>

#ifdef TAGLIB
#include <taglib/mpegfile.h>
#include <taglib/tag.h>
#endif /* TAGLIB */

#include <unistd.h>
#include <string.h>

cPostproc Postproc;

/* --- cPostproc -------------------------------------------------------------*/

cPostproc::cPostproc()
:cThread()
{
        convert = NULL;
        postdata = NULL;
        postdata_tail = NULL;
        
        file_buf.length = KB(10);
        file_buf.data = new uchar[file_buf.length];
        
        num_queued = 0;
}


cPostproc::~cPostproc()
{
        while (postdata)
                set_next_postdata();
        
        DELETE(convert);
        DELETEA(file_buf.data);
}


void cPostproc::Activate(bool on)
{
        if (on)
                Start();
        else
                Cancel(3);
}


void cPostproc::Action(void)
{
        dsyslog("[audiorecorder]: postprocessing thread started (%s, %s())",
                __FILE__, __func__);
        
        while (Running()) {
                if (postdata == NULL) {
                        sleep(1);
                        continue;
                }
                
                if (postdata->get_frames_fade_in() > 0 ||
                    postdata->get_frames_fade_out() > 0 ||
                    strcmp(postdata->get_codec(), "mp2") != 0) {
                        convert = new cConvert(postdata);
                        reencode();
                } else
                        rename_file();
                
#ifdef TAGLIB
                set_tag();
#endif /*TAGLIB */
                
                set_next_postdata();
                
                DELETE(convert);
        }
        
        dsyslog("[audiorecorder]: postprocessing thread stopped (%s, %s())",
                __FILE__, __func__);
}


void cPostproc::put_postdata(const cPostData *_postdata)
{
        cPostData *tmp = new cPostData(_postdata);
        
        Lock();
        
        if (! postdata_tail) {
                postdata = tmp;
                postdata_tail = tmp;
        }
        else {
                postdata_tail->set_next(tmp);
                postdata_tail = tmp;
        }
        
        num_queued++;
        
        Unlock();
        
        if (! Running())
                Activate(true);
}


void cPostproc::set_next_postdata(void)
{
        if (! postdata)
                return;
        
        Lock();
        
        cPostData *tmp = postdata->get_next();
        
        DELETE(postdata);
        postdata = tmp;
        
        if (! postdata)
                postdata_tail = NULL;
        
        num_queued--;
        
        Unlock();
}


void cPostproc::reencode(void)
{
        FILE *file_in, *file_out;
        abuffer *encoder_buf = NULL;
        long file_pos = 0;
        float volume = 0;
        int frames, frame = 0;
        
        if (! postdata->get_new_path() || ! postdata->get_path())
                return;
        
        if (access(postdata->get_new_path(), F_OK) != -1) {
                remove(postdata->get_path());
      		dsyslog("[audiorecorder]: track (%s) already exists (%s, "
                        "%s())", postdata->get_path(), __FILE__, __func__);
                return;
        }
        
        file_in = fopen(postdata->get_path(), "r");
        if (! file_in)
                return;
        
        file_out = fopen(postdata->get_new_path(), "w");
        if (! file_out)
                return;
        
        fseek(file_in, 0, SEEK_END);
        file_pos = ftell(file_in);
        frames = file_pos / postdata->get_len_mpa_frame();
        rewind(file_in);
        
        dsyslog("[audiorecorder]: start reencoding (%s into %s) (%s, %s())",
                postdata->get_path(), postdata->get_new_path(), __FILE__,
                __func__);
        dsyslog("[audiorecorder]: fade in mode: %d, %d frames, fade out mode: "
                "%d, %d frames (%s, %s())",
                postdata->get_fade_in_mode(), postdata->get_frames_fade_in(),
                postdata->get_fade_out_mode(), postdata->get_frames_fade_out(),
                __FILE__, __func__);
        
        file_pos = 0;
        file_buf.offset = 0;
        
        while (feof(file_in) == 0) {
                fseek(file_in, file_pos + file_buf.offset, SEEK_SET);
                file_pos = ftell(file_in);
                file_buf.offset = 0;
                
                fread(file_buf.data, sizeof(uchar), file_buf.length, file_in);
                
                get_mpa_frame(&file_buf, &mpa_frame, postdata->get_path());
                
                while(mpa_frame.data) {
                        file_buf.offset += mpa_frame.length;
                        
                        volume = get_volume(frame, frames);
                        encoder_buf = convert->reencode_mpa_frame(&mpa_frame,
                                volume);
                        
                        if (encoder_buf->data) {
                                fwrite(encoder_buf->data, sizeof(uchar),
                                        encoder_buf->offset, file_out);
                                /*
                                 * encoder_buf->offset is used to save the
                                 * size of the encoded frame ...
                                 */                                
                                frame++;
                        }
                        
                        get_mpa_frame(&file_buf, &mpa_frame,
                                postdata->get_path());
                }
        }
        
        FCLOSE(file_in);
        FCLOSE(file_out);
        
        remove(postdata->get_path());
        
        dsyslog("[audiorecorder]: stop reencoding (%s into %s) (%s, %s())",
                postdata->get_path(), postdata->get_new_path(), __FILE__,
                __func__);
}


float cPostproc::get_volume(int _frame, int _frames)
{
        int fade;
        
        if (_frame < postdata->get_frames_fade_in()) {
                fade = postdata->get_frames_fade_in();
                
                if (postdata->get_fade_in_mode() == 1)
                        return (float)_frame / (float)fade;
                else if (postdata->get_fade_in_mode() == 2)
                        return (float)(_frame * _frame) / (float)(fade * fade);
        }
        else if (_frame > _frames - postdata->get_frames_fade_out()) {
                fade = postdata->get_frames_fade_out();
                _frame = _frames - _frame;
                
                if (postdata->get_fade_out_mode() == 1)
                        return (float)_frame / (float)fade;
                else if (postdata->get_fade_out_mode() == 2)
                        return (float)(_frame * _frame) / (float)(fade * fade);
        }
        
        return 1;
}


#ifdef TAGLIB
void cPostproc::set_tag()
{
        TagLib::MPEG::File file(postdata->get_new_path());
        
        TagLib::Tag *tag = file.tag();
        
        tag->setArtist(postdata->get_artist());
        tag->setTitle(postdata->get_title());
        if (postdata->get_genre())
                tag->setGenre(postdata->get_genre());
        if (postdata->get_comment())
                tag->setComment(postdata->get_comment());
        
        file.save();
        
        dsyslog("[audiorecorder]: tag written (%s) (%s, %s())",
                postdata->get_new_path(), __FILE__, __func__);
}
#endif /* TAGLIB */


void cPostproc::rename_file()
{
        if (access(postdata->get_new_path(), F_OK) != -1) {
                remove(postdata->get_path());
      		dsyslog("[audiorecorder]: track (%s) already exists (%s, "
                        "%s())", postdata->get_new_path(), __FILE__, __func__);
                return;
        }
        
      	rename(postdata->get_path(), postdata->get_new_path());
        
     	dsyslog("[audiorecorder]: renamed %s to %s (%s, %s())",
                postdata->get_path(), postdata->get_new_path(), __FILE__,
                __func__);
}
