/*
 * rds.c
 */

#include "rds.h"
#include "audiorecorder.h"

#include <vdr/tools.h>

#include <string.h>

/*
 * rt-translation: 0x80..0xff
 * this table is taken from the vdr-radio-plugin, thx a lot ...
 */
const uchar rt_trans[128] = {
        0xe1, 0xe0, 0xe9, 0xe8, 0xed, 0xec, 0xf3, 0xf2,
        0xfa, 0xf9, 0xd1, 0xc7, 0x8c, 0xdf, 0x8e, 0x8f,
        0xe2, 0xe4, 0xea, 0xeb, 0xee, 0xef, 0xf4, 0xf6,
        0xfb, 0xfc, 0xf1, 0xe7, 0x9c, 0x9d, 0x9e, 0x9f,
        0xaa, 0xa1, 0xa9, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7,
        0xa8, 0xa9, 0xa3, 0xab, 0xac, 0xad, 0xae, 0xaf,
        0xba, 0xb9, 0xb2, 0xb3, 0xb1, 0xa1, 0xb6, 0xb7,
        0xb5, 0xbf, 0xf7, 0xb0, 0xbc, 0xbd, 0xbe, 0xa7,
        0xc1, 0xc0, 0xc9, 0xc8, 0xcd, 0xcc, 0xd3, 0xd2,
        0xda, 0xd9, 0xca, 0xcb, 0xcc, 0xcd, 0xd0, 0xcf,
        0xc2, 0xc4, 0xca, 0xcb, 0xce, 0xcf, 0xd4, 0xd6,
        0xdb, 0xdc, 0xda, 0xdb, 0xdc, 0xdd, 0xde, 0xdf,
        0xc3, 0xc5, 0xc6, 0xe3, 0xe4, 0xdd, 0xd5, 0xd8,
        0xde, 0xe9, 0xea, 0xeb, 0xec, 0xed, 0xee, 0xf0,
        0xe3, 0xe5, 0xe6, 0xf3, 0xf4, 0xfd, 0xf5, 0xf8,
        0xfe, 0xf9, 0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff };

/* --- cRds ----------------------------------------------------------------- */


cRds::cRds(const char *_channel_name, cPostData *_postdata)
{
        channel_name = strdup0(_channel_name);
        postdata = _postdata;
        recstat = recWait;
        
        buf.data = new uchar[RDS_BUF_SIZE];
        buf.length = 0;
        buf.offset = 0;
        
        rt_length = 0;
        lb0xfd = false;
        
        last_tb = -1;
        last_rb = -1;
}


cRds::~cRds()
{
        FREE(channel_name);
        DELETEA(buf.data);
}


void cRds::put_data(uchar *data, int length)
{
        int c, pos, len;
        
        pos = length - 1;
        len = data[pos - 1]; /* length of rds data */
        
        if (data[pos] != 0xfd || len == 0)
                return;
        
        if (buf.length + len >= RDS_BUF_SIZE) {
                dsyslog("[audiorecorder]: buffer overflow <%s> (%s, %s())",
                        channel_name, __FILE__, __func__);
                buf.length = 0;
                buf.offset = 0;
        }
                
        /* reverse rds data */
        for (c = 2; c < len + 2; c++) {
                
                /* byte stuffing */
                int bs = data[pos - c];
                
                if (bs == 0xfd) {
                        lb0xfd = true;
                        continue;
                }
                
                if (lb0xfd) {
                        switch (bs) {
                        case 0x00:
                                bs = 0xfd;
                                break;
                        case 0x01:
                                bs = 0xfe;
                                break;
                        default:
                                bs = 0xff;
                        }
                        
                        lb0xfd = false;
                }
                
                /* copy rds value on the buffer */
                buf.data[buf.length] = bs;
                buf.length++;
        }
}


bool cRds::set_next_frame(void)
{
        int offset;
        
        offset = buf.offset;
        rds_frame.data = NULL;
        rds_frame.length = 0;
        
        for (; buf.offset < buf.length - 4; buf.offset++) {
                if (buf.data[buf.offset] == 0xfe) {
                        /* rds start marker found */
                        rds_frame.length = buf.data[buf.offset + 4] + 8;
                        
                        if (buf.offset + rds_frame.length > buf.length)
                                break;
                        
                        rds_frame.data = buf.data + buf.offset;
                        
                        /* check rds end marker */
                        if (rds_frame.data[rds_frame.length - 1] != 0xff) {
                                dsyslog("[audiorecorder]: no rds end marker "
                                        "found <%s> (%s, %s())", channel_name,
                                        __FILE__, __func__);
                                print_buffer();
                        }
                        
                        break;
                }
        }
        
        if (buf.offset != offset && cPluginAudiorecorder::get_dbg_level() > 0) {
                printf("skipped %d byte(s) <%s> (%s, %s())\n", buf.offset -
                        offset, channel_name,  __FILE__, __func__);
                print_buffer();
        }
        
        if (! rds_frame.data) {
                delete_data(buf.offset);
                return false;
        }
        
        buf.offset += rds_frame.length;
        
        return true;
}


void cRds::delete_data(int length)
{
        /* clear the buffer */
        
        if (length < 1)
                return;
        
        buf.length -= length;
        buf.offset -= length;
        memmove(buf.data, buf.data + length, buf.length);
}


eRecStat cRds::decode_frame(void)
{
        /* reset recording status */
        switch (recstat) {
        case recNew:
                recstat = recRun;
                break;
        case recStop:
                recstat = recOff;
                break;
        default:
                break;
        }
        
        
        if (rds_frame.data[5] == mecRT)
                decode_radiotext();
        else if (rds_frame.data[5] == mecODA && rds_frame.data[7] == 0x4b &&
                 rds_frame.data[8] == 0xd7)
                decode_rtp();
        else if (rds_frame.data[5] == mecPTY) {
                if (! postdata->get_genre())
                        postdata->set_genre(rds_frame.data[8]);
                
                if (cPluginAudiorecorder::get_dbg_level() > 1)
                        printf("pty-code <%s>: %d\n", channel_name,
                                rds_frame.data[8]);
        }
        
        return recstat;
}


void cRds::decode_radiotext(void)
{
        int c, rt_ab_flag;
        
        rt_length = rds_frame.data[8] - 1;
        rt_ab_flag = rds_frame.data[9] & 0x01;
        
        for (c = 0; c < rt_length; c++) {
                if (rds_frame.data[c + 10] >= 0x80)
                        rds_frame.data[c + 10] =
                                rt_trans[(rds_frame.data[c + 10] - 0x80)];
                
                radiotext[c] = rds_frame.data[c + 10];
        }
        
        radiotext[rt_length] = '\0';
        
        if (cPluginAudiorecorder::get_dbg_level() > 1)
                printf("radiotext (%d) <%s>: %s\n", rt_ab_flag, channel_name,
                        radiotext);
}


void cRds::decode_rtp(void)
{
        int rb, tb;
        
        bool toggle_tb = false;
        bool toggle_rb = false;
        
        tb = (rds_frame.data[10] >> 4) & 0x01;
        if (last_tb == -1)
                last_tb = tb;
        
        rb = (rds_frame.data[10] >> 3) & 0x01;
        if (last_rb == -1)
                last_rb = rb;
        
        if (cPluginAudiorecorder::get_dbg_level() > 1)
                printf("rtp-data <%s>: toggle bit: %d, running bit: %d\n",
                       channel_name, tb , rb);
        
        if (last_tb != tb)
                toggle_tb = true;
        
        if (last_rb != rb)
                toggle_rb = true;
        
        last_tb = tb;
        last_rb = rb;
        
        if (recstat == recWait) {
                if (! toggle_tb && ! toggle_rb)
                        return;
                                
                /* ready to record */
                recstat = recOff;
        }
        
        if (rb == 1) {
                /* running bit is on */
                if (recstat == recOff) {
                        recstat = recNew;
                        rt_length = 0;
                }
                else if (toggle_tb) {
                        recstat = recStop;
                        return;
                }
        }
        else {
                /* running bit is off */
                if (recstat == recRun)
                        recstat = recStop;
                
                return;
        }
        
        if (recstat == recRun && rt_length > 0)
                decode_rtp_items();
}


void cRds::decode_rtp_items(void)
{
        int c, t[2], s[2], l[2];
        
        /* tag 1 */
        t[0] = ((rds_frame.data[10] << 3) & 0x38) | (rds_frame.data[11] >> 5);
        s[0] = ((rds_frame.data[11] << 1) & 0x3e) | (rds_frame.data[12] >> 7);
        l[0] = ((rds_frame.data[12] >> 1) & 0x3f) + 1;
        
        /* tag 2*/
        t[1] = ((rds_frame.data[12] << 5) & 0x20) | (rds_frame.data[13] >> 3);
        s[1] = ((rds_frame.data[13] << 3) & 0x38) | (rds_frame.data[14] >> 5);
        l[1] = (rds_frame.data[14] & 0x1f) + 1;
        
        for (c = 0; c < 2; c++) {
                if (cPluginAudiorecorder::get_dbg_level() > 1)
                        printf("rtp-data <%s>: type: %d, start: %d, length: "
                               "%d\n", channel_name, t[c], s[c], l[c]);
                
                if (t[c] < 1 || t[c] > 10)
                        continue;
                
                if (t[c] == ItemTitle) {
                        if (! postdata->get_title() &&
                            correct_rtp_tag(t[c], s[c], l[c]))
                                postdata->set_title(radiotext + s[c]);
                }
                else if (t[c] == ItemArtist) {
                        if (! postdata->get_artist() &&
                            correct_rtp_tag(t[c], s[c], l[c]))
                                postdata->set_artist(radiotext + s[c]);
                }
                else if (cPluginAudiorecorder::get_dbg_level() > 0) {
                        printf("rtp-data <%s>: !!! unsupported item %d found "
                               "!!!\n", channel_name, t[c]);
                }
        }
}


bool cRds::correct_rtp_tag(int &type, int &start, int &length)
{
        int end, length_org;
        
        if (start + length > rt_length) {
                length_org = length;
                
                length = rt_length - start;
                if (length < 0)
                        length = 0;
                
                if (cPluginAudiorecorder::get_dbg_level() > 1)
                        printf("rtp-data <%s>: got buggy tag-infos, shortened "
                               "length from %d to %d\n", channel_name,
                               length_org, length);
        }
        
        /* remove ' ' or '"' at the beginning of the tag */
        end = start + length;
        for (; start < end; start++) {
                if (radiotext[start] != ' ' && radiotext[start] != '"')
                        break;
                
        length--;
        }
        
        /* remove ' ' or '"'  at the end of the tag */
        for (; length > 0; length--) {
                if (radiotext[start + length - 1] != ' ' && radiotext[start +
                    length - 1] != '"')
                        break;
        }
        
        if (length <= 1 && cPluginAudiorecorder::get_dbg_level() > 1) {
                printf("rtp-data <%s>: got buggy tag-infos, length is too "
                       "short !\n", channel_name);
                return false;
        }
        
        radiotext[start + length] = '\0';
        
        return true;
}


void cRds::print_buffer(void)
{
        int c;
        
        if (! cPluginAudiorecorder::get_dbg_level() > 0)
                return;
        
        printf("rds-buffer (%d, %d) <%s>: ", buf.length, buf.offset,
                channel_name);
        
        for (c = 0; c < buf.length; c++)
                printf("%02x ", buf.data[c]);
        
        printf("\n");
}
